using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;

namespace Gdal
{
    public enum Access
    {
        ReadOnly = 0,
        Update = 1,
    }

    public class Dataset : IDisposable
    {
        internal HandleRef hDataset;

        /// <summary>
        /// Construct a Dataset that is associated with an existing file.
        /// </summary>
        /// <param name="filename">Filename to open</param>
        /// <param name="access">Access mode with which to open file</param>
        public Dataset(string filename, Access access)
        {
            hDataset = new HandleRef(this, InterOp.GDALOpen(filename, access));
            if (!IsValid)
                throw new IOException("Unable to open file " + filename + "\n" +
                    InterOp.CPLGetLastErrorMsg());
        }

        /// <summary>
        /// Construct a Dataset that is associated with an existing file, with ReadOnly access.
        /// </summary>
        /// <param name="filename"></param>
        public Dataset(string filename) : this(filename, Access.ReadOnly) { }

        internal Dataset(IntPtr ptrDataset)
        {
            hDataset = new HandleRef(this, ptrDataset);
        }

        /// <summary>
        /// True if the Dataset was successfully opened / created and has not been closed.
        /// </summary>
        public bool IsValid { get { return hDataset.Handle != IntPtr.Zero; } }

        /// <summary>
        /// The filename associated with this Dataset.
        /// </summary>
        public string Filename { get { return InterOp.GDALGetDescription(hDataset); } }

        /// <summary>
        /// The access with which this Dataset was opened.
        /// </summary>
        public Access Access { get { return (Access)InterOp.GDALGetAccess(hDataset); } }

        /// <summary>
        /// Close the Dataset. This method is also called by Dispose(). Do not invoke any methods
        /// on a Dataset after closing it!
        /// </summary>
        public void Close()
        {
            if (hDataset.Handle != IntPtr.Zero) {
                InterOp.GDALClose(hDataset);
                hDataset = new HandleRef(this, IntPtr.Zero);
            }
        }

        /// <summary>
        /// The width of the image.
        /// </summary>
        public int XSize { get { return InterOp.GDALGetRasterXSize(hDataset); } }

        /// <summary>
        /// The height of the image.
        /// </summary>
        public int YSize { get { return InterOp.GDALGetRasterYSize(hDataset); } }

        /// <summary>
        /// The width and height as a System.Drawing.Size object.
        /// </summary>
        public System.Drawing.Size ImageSize { get { return new System.Drawing.Size(XSize, YSize); } }

        /// <summary>
        /// The number of bands in the image.
        /// </summary>
        public int RasterCount { get { return InterOp.GDALGetRasterCount(hDataset); } }

        /// <summary>
        /// Return a RasterBand object for the i'th band. The first band has index 1.
        /// </summary>
        /// <param name="i"></param>
        /// <returns></returns>
        public RasterBand GetRasterBand(int i)
        {
            if (i > RasterCount || i < 1)
                throw new ApplicationException("Gdal.Dataset.GetRasterBand() invoked with invalid index");
            return new RasterBand(InterOp.GDALGetRasterBand(hDataset, i));
        }

        /// <summary>
        /// Fetch the affine transformation coefficients
        /// </summary>
        /// <remarks>
        /// Fetches the coefficients for transforming between pixel/line (X,Y) raster space, and projection coordinates (Xp,Yp) space.<br/>
        /// Xp = T[0] + T[1]*X + T[2]*Y<br/>
        /// Yp = T[3] + T[4]*X + T[5]*Y<br/>
        /// In a north up image, T[1] is the pixel width, and T[5] is the pixel height.
        /// The upper left corner of the upper left pixel is at position (T[0],T[3]).
        /// </remarks>
        /// <returns>Affine transformation parameters</returns>
        public GeoTransform GetGeoTransform()
        {
            double[] rawTransform = new double[6];
            if (InterOp.GDALGetGeoTransform(hDataset, rawTransform) == CPLErr.None)
                return new GeoTransform(rawTransform);
            else
                return new GeoTransform();
        }

        /// <summary>
        /// Set the affine transformation coefficients for this dataset.
        /// </summary>
        /// <param name="transform"></param>
        public void SetGeoTransform(GeoTransform transform)
        {
            // Silently ignore errors for now
            InterOp.GDALSetGeoTransform(hDataset, transform.transform);
        }

        /// <summary>
        /// Get the string describing the projection used by this dataset.
        /// </summary>
        /// <returns></returns>
        public string GetProjection()
        {
            return InterOp.GDALGetProjectionRef(hDataset);
        }

        /// <summary>
        /// Get the Driver object that handles files of the same format as this Dataset.
        /// </summary>
        /// <returns></returns>
        public Driver GetDriver()
        {
            return new Driver(InterOp.GDALGetDatasetDriver(hDataset));
        }

        public Dictionary<string, string> GetMetadata()
        {
            return InterOp.GDALGetMetadata(hDataset, null);
        }

        public string GetMetadataItem(string key)
        {
            return InterOp.GDALGetMetadataItem(hDataset, key, null);
        }

        /// <summary>
        /// Close the Dataset. Do not invoke any methods on this Dataset after calling Dispose()!
        /// </summary>
        /// <param name="disposing"></param>
        protected void Dispose(bool disposing)
        {
            if (disposing) {
                // dispose managed resources
            }
            Close();
        }

        /// <summary>
        /// Close the Dataset. Do not invoke any methods on this Dataset after calling Dispose()!
        /// </summary>
        /// <param name="disposing"></param>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~Dataset()
        {
            Dispose(false);
        }

    }
}
