using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace GDALTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            updateBitmapTimer.Tick += new EventHandler(updateBitmapTimer_Tick);
        }

        private string imageFilename = "";
        private Bitmap bitmap;
        private Timer updateBitmapTimer = new Timer();

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            if (dialog.ShowDialog() == DialogResult.OK) {
                imageFilename = dialog.FileName;
                UpdateBitmap();
                using (Gdal.Dataset dataset = new Gdal.Dataset(imageFilename)) {
                    // Setup info panel
                    filenameLabel.Text = dataset.Filename;
                    sizeLabel.Text = dataset.XSize + "x" + dataset.YSize;
                    Gdal.GeoTransform tform = dataset.GetGeoTransform();
                    transformLabel.Text = tform[0] + "," + tform[1] + "," + tform[2] + "," +
                        tform[3] + "," + tform[4] + "," + tform[5];
                    projLabel.Text = dataset.GetProjection();
                    // Setup metadata panel
                    Dictionary<string, string> metadata = dataset.GetMetadata();
                    metaDataListView.Items.Clear();
                    foreach (KeyValuePair<string, string> keyValue in metadata) {
                        ListViewItem item = new ListViewItem();
                        item.Text = keyValue.Key;
                        item.SubItems.Add(keyValue.Value);
                        metaDataListView.Items.Add(item);
                    }
                    // Resize the colums if we have any metadata
                    if (metaDataListView.Items.Count > 0) {
                        metaDataListView.Columns[0].Width = -1;
                        metaDataListView.Columns[1].Width = -1;
                    }
                }
            }
        }

        private void UpdateBitmap()
        {
            if (imageFilename.Length > 0) {
                using (Gdal.Dataset dataset = new Gdal.Dataset(imageFilename)) {
                    // Get image and drawing surface size
                    int width = dataset.XSize;
                    int height = dataset.YSize;
                    int visibleWidth = imageTab.Width;
                    int visibleHeight = imageTab.Height;
                    // Determine band assignments
                    int[] channels = GuessChannels(dataset);
                    bool isRGB = channels.Length == 3;
                    // Create new bitmap
                    bitmap = new Bitmap(visibleWidth, visibleHeight, 
                        isRGB ? PixelFormat.Format24bppRgb : PixelFormat.Format16bppGrayScale);
                    BitmapData bitmapData = bitmap.LockBits(new Rectangle(0, 0, visibleWidth, visibleHeight),
                            ImageLockMode.ReadWrite, bitmap.PixelFormat);
                    try {
                        int pixelStride = isRGB ? 3 : 1;
                        for (int i = 0; i < channels.Length; ++i) {
                            Gdal.RasterBand band = dataset.GetRasterBand(channels[i]);
                            IntPtr scan0 = new IntPtr(bitmapData.Scan0.ToInt64() + i);
                            band.RasterIO(Gdal.RWFlag.Read, 0, 0, width, height, scan0,
                                visibleWidth, visibleHeight, Gdal.DataType.Byte, pixelStride, bitmapData.Stride);
                        }
                    }
                    finally {
                        bitmap.UnlockBits(bitmapData);
                    }
                    imageTab.Invalidate();
                }
            }
        }

        private void updateBitmapTimer_Tick(object o, EventArgs a)
        {
            updateBitmapTimer.Enabled = false;
            UpdateBitmap();
        }


        // Setup the channels array depending on the contents of the dataset. If in RGB mode,
        // then channels[0] contains the blue component, channels[1], the green component, and
        // channels[2] the red component.
        private int[] GuessChannels(Gdal.Dataset dataset)
        {
            int nBands = dataset.RasterCount;
            int[] channels;
            if (nBands == 1 || nBands == 2) {
                // If less than three bands just display first band as grayscale
                channels = new int[1];
                channels[0] = 1;
            }
            else {
                // Assume RGB image
                channels = new int[3];
                // If three bands, then assume bands are in RGB order. If more than three,
                // assume bands are in order of increasing wavelength (BGR...)
                if (nBands == 3) {
                    channels[0] = 3;
                    channels[1] = 2;
                    channels[2] = 1;
                }
                else {
                    channels[0] = 1;
                    channels[1] = 2;
                    channels[2] = 3;
                }
                // Now check to see if any bands actually have ColorInterpretations. If
                // they do, this will override the defaults.
                for (int i = 1; i <= nBands; ++i) {
                    Gdal.ColorInterp c = dataset.GetRasterBand(i).ColorInterpretation;
                    if (c == Gdal.ColorInterp.RedBand)
                        channels[2] = i;
                    else if (c == Gdal.ColorInterp.GreenBand)
                        channels[1] = i;
                    else if (c == Gdal.ColorInterp.BlueBand)
                        channels[0] = i;
                }
            }
            return channels;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void imageTab_Paint(object sender, PaintEventArgs e)
        {
            if (bitmap != null) {
                Graphics g = e.Graphics;
                g.DrawImage(bitmap, new Point(0, 0));
            }
        }

        private void imageTab_Resize(object sender, EventArgs e)
        {
            // Don't update the bitmap immediately since this might be slow, 
            // instead, start the updateBitmapTimer to update the bitmap in 500ms time,
            // if we don't get another resize event before then.
            updateBitmapTimer.Interval = 500;
            updateBitmapTimer.Start();
        }
    }
}